/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Ce = Components.Exception;
const Cu = Components.utils;

const CLASS_ID = Components.ID("{fd1c6482-a731-4b2c-b384-7510632c0bbd}");
const CLASS_NAME = "Daap Auth Prompt";
const CONTRACT_ID = "@songbirdnest.com/daap/auth-prompt;1";

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

/* global variables -----------------------------------------------------------*/
var l10n = null;
var promptService = null;
var loginManager = null;


const PROPERTY_FILE = "chrome://daapclient/locale/daapclient.properties";

/*=========================================================================*/
/* SbDaapAuthPrompt                                                        */
/*=========================================================================*/

/* SbDaapAuthPrompt -------------------------------------------------------*/
function SbDaapAuthPrompt() 
{ 
  /* strings */
  if (!(l10n instanceof Ci.nsIStringBundleService)) {
    l10n = Cc["@mozilla.org/intl/stringbundle;1"]
            .getService(Ci.nsIStringBundleService)
            .createBundle(PROPERTY_FILE);
  }
  /* prompt service */
  if (!(promptService instanceof Ci.nsIPromptService)) {
    promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"]
               .getService(Ci.nsIPromptService);
  }
  /* login manager */
  if (!(loginManager instanceof Ci.nsILoginManager)) {
    loginManager = Cc["@mozilla.org/login-manager;1"]
                    .getService(Ci.nsILoginManager);
  }

  this._requestor = null;
}


/* SbDaapAuthPrompt implementation ----------------------------------------*/
SbDaapAuthPrompt.prototype = {

  /* nsISupports ---------------------------------------------------------*/
  QueryInterface: XPCOMUtils.generateQI( [Ci.nsIAuthPrompt2, Ci.nsIClassInfo]),

  
  /* nsIClassInfo, so callers don't have to QI from JS -------------------*/
  classDescription        : CLASS_NAME,
  classID                 : CLASS_ID,
  contractID              : CONTRACT_ID,
  flags                   : Ci.nsIClassInfo.THREADSAFE,
  implementationLanguage  : Ci.nsIProgrammingLanguage.JAVASCRIPT,
  getHelperForLanguage    : function(aLanguage) { return null; },
  getInterfaces : function (count) {
    var interfaces = [Ci.nsIAuthPrompt2, Ci.nsIClassInfo, Ci.nsISupports];
    count.value = interfaces.length;
    return interfaces;
  },


  /* nsIAuthPrompt2 -------------------------------------------------------*/

  /* nsICancelable asyncPromptAuth ( nsIChannel channel , nsIAuthPromptCallback callback , nsISupports context , PRUint32 level , nsIAuthInformation authInfo ) */
  asyncPromptAuth: function SbDaapAuthPrompt_asyncPromptAuth (aChannel, aCallback, aContext, aLevel, authInfo)
  {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },

  /* PRBool promptAuth ( nsIChannel channel , PRUint32 level , nsIAuthInformation authInfo ) */
  promptAuth: function SbDaapAuthPrompt_promptAuth (aChannel, aLevel, authInfo)
  {

    if (!(aChannel instanceof Ci.nsIHttpChannel)) {
      throw Components.results.NS_ERROR_INVALID_ARG;
    }

    /* get the main window */
    if (typeof(window) == "undefined") { 
      var window = Cc["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Ci.nsIWindowMediator)
                    .getMostRecentWindow("Songbird:Main").window;
    }

    let title = l10n.GetStringFromName("DlgErrorTitle");
    let message = null;

    
    //TODO: identify the library by ID (not with the current URL)
    let url = aChannel.originalURI.resolve("/");
    let password = {value: ""};
    let checkMessage = l10n.GetStringFromName("RememberPasswordCheckMessage");
    let check = {value: false};

    let nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
                                                Components.interfaces.nsILoginInfo, "init");
    
    /* Find users for the given parameters */
    let logins = loginManager.findLogins({}, "chrome://daapclient", url, null);
    let login = null;
      
    /* Find user from returned array of nsILoginInfo objects */
    if (logins.length) {
      login = logins[0];
      password.value = login.password;
      check.value = true;
    }
    
    let ok = promptService.promptPassword(window, 
                                    l10n.GetStringFromName("Authentication"), 
                                    l10n.GetStringFromName("Password")+":", 
                                    password, checkMessage, check);

    if (ok) {
      if (check.value) {
        /* store the password */
        let loginInfo = new nsLoginInfo("chrome://daapclient", url, null,
                                        "daap", password.value, "", "");
        if (login) {
          if (login.password != password.value) {
            loginManager.modifyLogin(login, loginInfo);
          }
        }
        else {
          loginManager.addLogin(loginInfo);
        }
      }
      else {
        /* remove all passwords */
        let logins = loginManager.findLogins({}, "chrome://daapclient", url, null);
      
        for (var i = 0; i < logins.length; i++) {
          loginManager.removeLogin(logins[i]);
        }
      }
          
      /* return the new password */
      authInfo.password = password.value;

      return true;
    }
    
    return false;
  }, //promptAuth

};


/* components constructors ================================================*/
var components = [SbDaapAuthPrompt];

/* NSGetModule entry point */
function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule(components);
}

