/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cu = Components.utils;
const Cr = Components.results;

const nsISupports = Components.interfaces.nsISupports;

const CLASS_ID = Components.ID("eb80d4d7-e609-4bd5-8f80-ae094bc220e8");
const CLASS_NAME = "Daap v3 Hash";
const CONTRACT_ID = "@songbirdnest.com/daap/cryptohash;1";

/* SbDaapCryptoHash constructor ------------------------------------------*/
function SbDaapCryptoHash() {
  this.initialized = false;

  this.state = new array(4);
  this.count = new array(2);
  this.count[0] = 0;
  this.count[1] = 0;
  this.buffer = new array(64);
  this.transformBuffer = new array(16);
  this.digestBits = new array(16);
}

/* SbDaapCryptoHash implementation ------------------------------------------*/
SbDaapCryptoHash.prototype = {

  /* nsISupports ------------------------------------------------------------*/
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(Ci.sbIDaapCryptoHash)
        && !aIID.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  },

  /* private data ---------------------------------------------------------*/
  initialized: false,

  state: null,
  count: null,
  buffer: null, 
  transformBuffer: null,
  digestBits: null,


  /* sbIDaapCryptoHash ----------------------------------------------------*/

  /* void init (in unsigned long aAlgorithm); */
  init: function(aAlgorithm) 
  {
    if (aAlgorithm != Ci.sbIDaapCryptoHash.MD5DAAP) {
      throw Cr.NS_ERROR_INVALID_ARG;
    }

    this.count[0] = 0;
    this.count[1] = 0;

    this.state[0] = 0x67452301;
    this.state[1] = 0xefcdab89;
    this.state[2] = 0x98badcfe;
    this.state[3] = 0x10325476;

    for (var i = 0; i < this.digestBits.length; i++) {
      this.digestBits[i] = 0;
    }

    this.initialized = true;
  },

  /* void initWithString (in ACString aAlgorithm); */
  initWithString: function(aAlgorithm)
  {
    if (aAlgorithm == "md5daap") {
      return this.init(Ci.sbIDaapCryptoHash.MD5DAAP);
    }

    throw Cr.NS_ERROR_INVALID_ARG;
  },

  /* void update ([array, size_is (aLen), const] in octet aData, 
   *               in unsigned long aLen); 
   **/
  update: function(aData, aLen)
  {
    if (!this.initialized) {
      throw Cr.NS_ERROR_NOT_INITIALIZED;
    }

    for (var i=0; i<aLen; i++) {
      this.updateByte(aData[i]);
    }
  },

  /* void updateFromStream (in nsIInputStream aStream, in unsigned long aLen); */
  updateFromStream: function(aStream, aLen)
  {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  },

  /* ACString finish (in PRBool aASCII); */
  finish: function(aASCII)
  {
    if (!this.initialized) {
      throw Cr.NS_ERROR_NOT_INITIALIZED;
    }

    if (aASCII) {
      return NS_ERROR_NOT_IMPLEMENTED;
    }
      
    var bits = new array(8);
    var padding;
    var i=0, j=3, index=0, padLen=0;

    for (i = 0; i < 4; i++) {
      bits[i] = and(shr(this.count[0],(i * 8)), 0xff);
    }
    for (i = 0; i < 4; i++) {
      bits[i+4]=and(shr(this.count[1],(i * 8)), 0xff);
    }
    index = and(shr(this.count[0], 3) ,0x3f);
    padLen = (index < 56) ? (56 - index) : (120 - index);
    padding = new array(64);
    padding[0] = 0x80;
    for (i=0;i<padLen;i++) {
      this.updateByte(padding[i]);
    }
    for (i=0;i<8;i++) {
      this.updateByte(bits[i]);
    }

    for (i = 0; i < 4; i++) {
      for (j = 0; j < 4; j++) {
        this.digestBits[i*4+j] = and(shr(this.state[i], (j * 8)) , 0xff);
      }
    }

    //XXX: binary data? XXX:
    var result = "";
    for(var i=0; i<this.digestBits.length; i++) {
      result += String.fromCharCode(this.digestBits[i]);
    }
    return result;
  },

  updateByte: function(b)
  {
    var index = and(shr(this.count[0],3) , 0x3f);
    if (this.count[0]<0xffffffff-7) {
      this.count[0] += 8;
    }
    else {
      this.count[1]++;
      this.count[0]-=0xffffffff+1;
      this.count[0]+=8;
    }
    this.buffer[index] = and(b,0xff);
    if (index  >= 63) {
      this.transform(this.buffer, 0);
    }
  },

  transform: function (buf, offset) 
  {
    var a=0, b=0, c=0, d=0;
    var x = this.transformBuffer;
    var i, j;

    a = this.state[0];
    b = this.state[1];
    c = this.state[2];
    d = this.state[3];

    for (i = 0; i < 16; i++) {
        x[i] = and(buf[i*4+offset],0xff);
        for (j = 1; j < 4; j++) {
            x[i]+=shl(and(buf[i*4+j+offset] ,0xff), j * 8);
        }
    }

    /* group 1 */
    a = FF ( a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */
    d = FF ( d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */
    c = FF ( c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
    b = FF ( b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
    a = FF ( a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
    d = FF ( d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
    c = FF ( c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
    b = FF ( b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
    a = FF ( a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
    d = FF ( d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */
    c = FF ( c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
    b = FF ( b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
    a = FF ( a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
    d = FF ( d, a, b, c, x[13], S12, 0xfd987193); /* 14 */
    c = FF ( c, d, a, b, x[14], S13, 0xa679438e); /* 15 */
    b = FF ( b, c, d, a, x[15], S14, 0x49b40821); /* 16 */

    /* group 2 */
    a = GG ( a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
    d = GG ( d, a, b, c, x[ 6], S22, 0xc040b340); /* 18 */
    c = GG ( c, d, a, b, x[11], S23, 0x265e5a51); /* 19 */
    b = GG ( b, c, d, a, x[ 0], S24, 0xe9b6c7aa); /* 20 */
    a = GG ( a, b, c, d, x[ 5], S21, 0xd62f105d); /* 21 */
    d = GG ( d, a, b, c, x[10], S22,  0x2441453); /* 22 */
    c = GG ( c, d, a, b, x[15], S23, 0xd8a1e681); /* 23 */
    b = GG ( b, c, d, a, x[ 4], S24, 0xe7d3fbc8); /* 24 */
    a = GG ( a, b, c, d, x[ 9], S21, 0x21e1cde6); /* 25 */
    d = GG ( d, a, b, c, x[14], S22, 0xc33707d6); /* 26 */
    c = GG ( c, d, a, b, x[ 3], S23, 0xf4d50d87); /* 27 */
    //b = GG ( b, c, d, a, x[ 8], S24, 0x455a14ed); /* 28 */  /* original */
    b = GG ( b, c, d, a, x[ 8], S24, 0x445a14ed); /* 28 */  /* apple md5 */
    a = GG ( a, b, c, d, x[13], S21, 0xa9e3e905); /* 29 */
    d = GG ( d, a, b, c, x[ 2], S22, 0xfcefa3f8); /* 30 */
    c = GG ( c, d, a, b, x[ 7], S23, 0x676f02d9); /* 31 */
    b = GG ( b, c, d, a, x[12], S24, 0x8d2a4c8a); /* 32 */

    /* group 3 */
    a = HH ( a, b, c, d, x[ 5], S31, 0xfffa3942); /* 33 */
    d = HH ( d, a, b, c, x[ 8], S32, 0x8771f681); /* 34 */
    c = HH ( c, d, a, b, x[11], S33, 0x6d9d6122); /* 35 */
    b = HH ( b, c, d, a, x[14], S34, 0xfde5380c); /* 36 */
    a = HH ( a, b, c, d, x[ 1], S31, 0xa4beea44); /* 37 */
    d = HH ( d, a, b, c, x[ 4], S32, 0x4bdecfa9); /* 38 */
    c = HH ( c, d, a, b, x[ 7], S33, 0xf6bb4b60); /* 39 */
    b = HH ( b, c, d, a, x[10], S34, 0xbebfbc70); /* 40 */
    a = HH ( a, b, c, d, x[13], S31, 0x289b7ec6); /* 41 */
    d = HH ( d, a, b, c, x[ 0], S32, 0xeaa127fa); /* 42 */
    c = HH ( c, d, a, b, x[ 3], S33, 0xd4ef3085); /* 43 */
    b = HH ( b, c, d, a, x[ 6], S34,  0x4881d05); /* 44 */
    a = HH ( a, b, c, d, x[ 9], S31, 0xd9d4d039); /* 45 */
    d = HH ( d, a, b, c, x[12], S32, 0xe6db99e5); /* 46 */
    c = HH ( c, d, a, b, x[15], S33, 0x1fa27cf8); /* 47 */
    b = HH ( b, c, d, a, x[ 2], S34, 0xc4ac5665); /* 48 */

    /* group 4 */
    a = II ( a, b, c, d, x[ 0], S41, 0xf4292244); /* 49 */
    d = II ( d, a, b, c, x[ 7], S42, 0x432aff97); /* 50 */
    c = II ( c, d, a, b, x[14], S43, 0xab9423a7); /* 51 */
    b = II ( b, c, d, a, x[ 5], S44, 0xfc93a039); /* 52 */
    a = II ( a, b, c, d, x[12], S41, 0x655b59c3); /* 53 */
    d = II ( d, a, b, c, x[ 3], S42, 0x8f0ccc92); /* 54 */
    c = II ( c, d, a, b, x[10], S43, 0xffeff47d); /* 55 */
    b = II ( b, c, d, a, x[ 1], S44, 0x85845dd1); /* 56 */
    a = II ( a, b, c, d, x[ 8], S41, 0x6fa87e4f); /* 57 */
    d = II ( d, a, b, c, x[15], S42, 0xfe2ce6e0); /* 58 */
    c = II ( c, d, a, b, x[ 6], S43, 0xa3014314); /* 59 */
    b = II ( b, c, d, a, x[13], S44, 0x4e0811a1); /* 60 */
    a = II ( a, b, c, d, x[ 4], S41, 0xf7537e82); /* 61 */
    d = II ( d, a, b, c, x[11], S42, 0xbd3af235); /* 62 */
    c = II ( c, d, a, b, x[ 2], S43, 0x2ad7d2bb); /* 63 */
    b = II ( b, c, d, a, x[ 9], S44, 0xeb86d391); /* 64 */

    this.state[0] +=a;
    this.state[1] +=b;
    this.state[2] +=c;
    this.state[3] +=d;
  }

}

/* helper functions -------------------------------------------------------*/
function array(n) {
  for(i=0;i<n;i++) this[i]=0;
  this.length=n;
}

function integer(n) { return n%(0xffffffff+1); }

function shr(a,b) {
  a=integer(a);
  b=integer(b);
  if (a-0x80000000>=0) {
    a=a%0x80000000;
    a>>=b;
    a+=0x40000000>>(b-1);
  } else
    a>>=b;
  return a;
}

function shl1(a) {
  a=a%0x80000000;
  if (a&0x40000000==0x40000000)
  {
    a-=0x40000000;
    a*=2;
    a+=0x80000000;
  } else
    a*=2;
  return a;
}

function shl(a,b) {
  a=integer(a);
  b=integer(b);
  for (var i=0;i<b;i++) a=shl1(a);
  return a;
}

function and(a,b) {
  a=integer(a);
  b=integer(b);
  var t1=(a-0x80000000);
  var t2=(b-0x80000000);
  if (t1>=0)
    if (t2>=0)
      return ((t1&t2)+0x80000000);
    else
      return (t1&b);
  else
    if (t2>=0)
      return (a&t2);
    else
      return (a&b);
}

function or(a,b) {
  a=integer(a);
  b=integer(b);
  var t1=(a-0x80000000);
  var t2=(b-0x80000000);
  if (t1>=0)
    if (t2>=0)
      return ((t1|t2)+0x80000000);
    else
      return ((t1|b)+0x80000000);
  else
    if (t2>=0)
      return ((a|t2)+0x80000000);
    else
      return (a|b);
}

function xor(a,b) {
  a=integer(a);
  b=integer(b);
  var t1=(a-0x80000000);
  var t2=(b-0x80000000);
  if (t1>=0)
    if (t2>=0)
      return (t1^t2);
    else
      return ((t1^b)+0x80000000);
  else
    if (t2>=0)
      return ((a^t2)+0x80000000);
    else
      return (a^b);
}

function not(a) {
  a=integer(a);
  return (0xffffffff-a);
}

/* md5 functions ---------------------------------------------------------*/

const S11 = 7;
const S12 = 12;
const S13 = 17;
const S14 = 22;
const S21 = 5;
const S22 = 9;
const S23 = 14;
const S24 = 20;
const S31 = 4;
const S32 = 11;
const S33 = 16;
const S34 = 23;
const S41 = 6;
const S42 = 10;
const S43 = 15;
const S44 = 21;

function F(x,y,z) {
  return or(and(x,y),and(not(x),z));
}

function G(x,y,z) {
  return or(and(x,z),and(y,not(z)));
}

function H(x,y,z) {
  return xor(xor(x,y),z);
}

function I(x,y,z) {
  return xor(y ,or(x , not(z)));
}

function rotateLeft(a,n) {
  return or(shl(a, n),(shr(a,(32 - n))));
}

function FF(a,b,c,d,x,s,ac) {
  a = a+F(b, c, d) + x + ac;
  a = rotateLeft(a, s);
  a = a+b;
  return a;
}

function GG(a,b,c,d,x,s,ac) {
  a = a+G(b, c, d) +x + ac;
  a = rotateLeft(a, s);
  a = a+b;
  return a;
}

function HH(a,b,c,d,x,s,ac) {
  a = a+H(b, c, d) + x + ac;
  a = rotateLeft(a, s);
  a = a+b;
  return a;
}

function II(a,b,c,d,x,s,ac) {
  a = a+I(b, c, d) + x + ac;
  a = rotateLeft(a, s);
  a = a+b;
  return a;
}

//=================================================
// Note: You probably don't want to edit anything
// below this unless you know what you're doing.
//
// Factory
var SbDaapCryptoHashFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new SbDaapCryptoHash()).QueryInterface(aIID);
  }
};

// Module
var SbDaapCryptoHashModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return SbDaapCryptoHashFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

//module initialization
function NSGetModule(aCompMgr, aFileSpec) { return SbDaapCryptoHashModule; }

