/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Ce = Components.Exception;
const Cu = Components.utils;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

//XXXdaniel: for removing the "SBDataSetStringValue" error messages
Components.utils.import("resource://app/jsmodules/SBDataRemoteUtils.jsm");


/*=========================================================================*/
/* DaapToXml Content Handler                                               */
/*=========================================================================*/

function XmlHandler()
{
  this.output = null;
  this.buffer = null;
}

XmlHandler.prototype ={
  output: null,
  buffer: null,

  startDocument: function() 
  {
    this.output = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n";
  },

  endDocument: function() {
  },

  startElement: function(aCode, aLength, aType) 
  {
    this.buffer = "";
    this.output += "<" + aCode + " type=\"" + aType +"\" "
                 + "length=\"" + aLength + "\">";
  },

  endElement: function(aCode, aLength, aType)
  {
    if (this.buffer && this.buffer.length) {
      this.output += "<![CDATA["+ this.buffer + "]]>"; 
    }
    this.buffer = null;
    this.output += "</" + aCode + ">";
  },

  characters: function(value) 
  {
    this.buffer += value;
  }

};


/*=========================================================================*/
/* SbDaapToXML                                                             */
/*=========================================================================*/

/* SbDaapToXML Constructor ------------------------------------------------*/
function SbDaapToXML() 
{ 
  this.listener = null;
  this.context = null;

  this.parser = null;
  this.handler = null;

  this.offset = null;
}

/* SbDaapToXML implementation ---------------------------------------------*/
SbDaapToXML.prototype = {

  /* nsISupports ---------------------------------------------------------*/
  QueryInterface: XPCOMUtils.generateQI( [Ci.nsIStreamConverter, 
      Ci.nsIRequestObserver, Ci.nsIStreamListener, Ci.nsIClassInfo]),


  /* nsIClassInfo, so callers don't have to QI from JS -------------------*/
  classDescription: "Daap Stream to XML Converter",
  classID:          Components.ID("{7092fe24-a582-43bf-a003-6e3d0555e4cf}"),
  contractID:       "@mozilla.org/streamconv;1?from=application/x-dmap-tagged&to=*/*",
  flags                   : Ci.nsIClassInfo.THREADSAFE,
  implementationLanguage  : Ci.nsIProgrammingLanguage.JAVASCRIPT,
  getHelperForLanguage    : function(aLanguage) { return null; },
  getInterfaces : function(count) {
    var interfaces = [Ci.nsIStreamConverter, Ci.nsIRequestObserver, 
                      Ci.nsIStreamListener, Ci.nsISupports, Ci.nsIClassInfo];
    count.value = interfaces.length;
    return interfaces;
  },


  /* private variables ----------------------------------------------------*/
  listener: null,
  context: null,

  parser: null,
  handler: null,

  offset: null,

  /* nsIStreamConverter ---------------------------------------------------*/

  /* nsIInputStream convert(in nsIInputStream aFromStream,
   *                        in string aFromType,
   *                        in string aToType,
   *                        in nsISupports aCtxt);
   */
  convert: function(aFromStream, aFromType, aToType, aContext)
  {
    if (!aFromStream) {
      throw Components.results.NS_ERROR_INVALID_ARG;
    }

    if (aFromType != "application/x-dmap-tagged") {
      /* don't do anything */
      return aFromStream;
    }

    /* add a buffer */
    var bufferedStream = Cc["@mozilla.org/network/buffered-input-stream;1"]
                          .createInstance(Ci.nsIBufferedInputStream);
    bufferedStream.init(aFromStream, 0x10000);

    var parser = Cc["@songbirdnest.com/daap/parser;2"]
                  .createInstance(Ci.sbIDaapParser);
    var handler = new XmlHandler();

    if (!parser || !handler) {
      throw Cr.NS_ERROR_FAILURE;
    }

    parser.parseFromStream(aFromStream, handler);

    /* send the output */
    var stringStream = Cc["@mozilla.org/io/string-input-stream;1"]
                        .createInstance(Ci.nsIStringInputStream);

    if (handler.output.length) {
      stringStream.setData(handler.output, handler.output.length);
    }

    return stringStream;
  },

  /* void asyncConvertData(in string aFromType,
   *                       in string aToType,
   *                       in nsIStreamListener aListener,
   *                       in nsISupports aCtxt);
   */
  asyncConvertData: function(aFromType, aToType, aListener, aContext)
  {
    /* initialize */
    this.listener = aListener;
    this.context = aContext;

    this.parser = Cc["@songbirdnest.com/daap/parser;2"]
                    .createInstance(Ci.sbIDaapParser);
    this.handler = new XmlHandler();

    if (!this.parser || !this.handler) {
      throw Cr.NS_ERROR_FAILURE;
    }
    this.parser.parseAsync(this.handler);

  },

  /* nsIRequestObserver ---------------------------------------------------*/

  /*  void onStartRequest ( nsIRequest request , nsISupports context ) */
  onStartRequest: function (aRequest, aContext)
  {
    /* initialize */
    this.offset = 0;

    var channel = aRequest.QueryInterface(Ci.nsIChannel);
    if (channel) {
      //application/x-dmap-tagged 
      //channel.contentType = "text/xml";
      channel.contentType = "text/plain";
      //this.originalURI = channel.originalURI;
    }

    let requestObserver = this.parser.QueryInterface(Ci.nsIRequestObserver);
    requestObserver.onStartRequest(aRequest, aContext);

    this.listener.onStartRequest(aRequest, this.context);
  },

  /*  void onStopRequest ( nsIRequest request , nsISupports context , 
   *                       nsresult statusCode ) 
   **/
  onStopRequest: function (aRequest, aContext, aStatusCode)
  {
    let requestObserver = this.parser.QueryInterface(Ci.nsIRequestObserver);
    requestObserver.onStopRequest(aRequest, aContext, aStatusCode);

    /* send the output */
    if (this.handler.output.length) {
      var buffer = this.handler.output;
      this.handler.output = "";

      var stringStream = Cc["@mozilla.org/io/string-input-stream;1"]
                          .createInstance(Ci.nsIStringInputStream);
      stringStream.setData(buffer, buffer.length);

      this.listener.onDataAvailable(aRequest, this.context, stringStream, 
                                    this.offset, buffer.length);

      this.offset += buffer.length;
    }

    this.listener.onStopRequest(aRequest, this.context, aStatusCode);

    /* clean */
    this.listener = null;
    this.context = null;
    this.offset = null;
    this.parser = null;
    this.handler = null;
  },

  /* nsIStreamListener ---------------------------------------------------*/
  
  /* void onDataAvailable ( nsIRequest request , nsISupports context , 
   *                        nsIInputStream inputStream , PRUint32 offset , 
   *                        PRUint32 count ) 
   **/
  onDataAvailable: function (aRequest, aContext, aInputStream, aOffset, aCount)
  {
    let streamListener = this.parser.QueryInterface(Ci.nsIStreamListener);
    streamListener.onDataAvailable(aRequest, aContext, aInputStream, aOffset, aCount);

    /* send the output */
    if (this.handler.output.length) {
      var buffer = this.handler.output;
      this.handler.output = "";
      var stringStream = Cc["@mozilla.org/io/string-input-stream;1"]
                          .createInstance(Ci.nsIStringInputStream);
      stringStream.setData(buffer, buffer.length);

      this.listener.onDataAvailable(aRequest, this.context, stringStream, 
            this.offset, buffer.length);

      this.offset += buffer.length;
    }

  }
        
}


/* components constructors ================================================*/
var components = [SbDaapToXML];

/* NSGetModule entry point */
function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule(components);
}

